﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Valentia.Kinection.Service.Manager
{
    /// <summary>
    /// This form is used to show all the saved notifications sent by Kinect windows service
    /// </summary>
    public partial class NotificationsHistory : Form
    {
        #region Declarations
        
        /// <summary>
        /// This event is used to clear notification history
        /// </summary>
        public event EventHandler ClearNotifications;

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="notificationsList">DataTable contains the notifications sent by Kinect windows service</param>
        public NotificationsHistory(DataTable notificationsList)
        {
            InitializeComponent();

            // Registering listview event
            this.lvwNotificationsHistory.DrawItem 
                += new DrawListViewItemEventHandler(OnDrawItem); 

            // Postioning the form in the center of the screen
            this.StartPosition = FormStartPosition.CenterScreen;

            BindListView(notificationsList);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// This method is used to bind notifications information table to list view
        /// </summary>
        /// <param name="notificationsList">DataTable contains the notifications sent by Kinect windows service</param>
        private void BindListView(DataTable notificationsList)
        {
            try
            {                
                foreach (DataRow notification in notificationsList.Rows)
                {
                    lvwNotificationsHistory.Items.Add(
                        new ListViewItem(new string[] { notification["NotificationDateTime"].ToString()
                        , notification["NotificationType"].ToString(), notification["NotificationMessage"].ToString()}));
                }
            }
            catch { }
        }

        #endregion

        #region Form Control Events

        /// <summary>
        /// Occurs when clear history button is clicked
        /// </summary>
        /// <param name="sender">Sender</param>
        /// <param name="e">Event Args</param>
        private void btnClear_Click(object sender, EventArgs e)
        {
            try
            {
                lvwNotificationsHistory.Items.Clear();

                if (ClearNotifications != null)
                {
                    ClearNotifications(sender, e);
                }
            }
            catch { }
        }

        /// <summary>
        /// Occurs when a System.Windows.Forms.ListView is drawn 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnDrawItem(object sender, DrawListViewItemEventArgs e)
        {
            try
            {
                foreach (ListViewItem.ListViewSubItem subItem in e.Item.SubItems)
                {
                    e.Item.UseItemStyleForSubItems = false;

                    subItem.BackColor = e.ItemIndex % 2 == 0
                        ? Color.FromArgb(209, 234, 251) : Color.FromArgb(189, 218, 238);

                    subItem.ForeColor = e.ItemIndex % 2 == 0
                        ? Color.FromArgb(0, 78, 152) : Color.FromArgb(0, 78, 152);

                    subItem.Font = e.Item.Font;
                }
            }
            catch { }

            e.DrawDefault = true;
        }

        #endregion
     }
}
